
-------------------------------------------------------------------------------
-- Optimzation
-------------------------------------------------------------------------------

-- GLOBALS: C_EditMode, EditModeManagerFrame, Enum, format

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Core Layout")

local LAYOUT_TO_PROFILE = {
	[C.Enum.Layout.Standard] = "SixUI",
	[C.Enum.Layout.Minimal] = "SixUI Minimal",
	[C.Enum.Layout.Damage] = "SixUI Damage",
}

local PROFILE_TO_LAYOUT = {
	["SixUI"] = C.Enum.Layout.Standard,
	["SixUI Minimal"] = C.Enum.Layout.Minimal,
	["SixUI Damage"] = C.Enum.Layout.Damage,
}

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	self.db = D.db:RegisterNamespace("Core Layout", {char = {layout = C.Enum.Layout.Standard}})

	A:AddCommand("layout", function (layout)
		if not layout or layout == "" then
			A.PrintF("You must specify a layout name by typing '%s'.", A.Highlight("/sixui layout [name]"))
		else
			self:SetLayout(layout)
		end
	end, "change SixUI's layout.")
end

function M:OnEnable()
	if not self:IsHooked(C_EditMode, "SetActiveLayout") then
		self:SecureHook(C_EditMode, "SetActiveLayout", "C_EditMode_SetActiveLayout")
	end

	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
	self:RegisterMessage("SIXUI_PROFILE_UPDATE")
end

-- Get the active layout,
function M:GetLayout()
	return D.db.profile.layout.activeLayout or C.Enum.Layout.Standard
end

A.GetLayout = M.GetLayout

-- Get a list of layouts for use in an options panel.
function M:GetOptions()
	return C.Enum.Layout
end

-- Get the active layout's profile.
function M:GetProfile()
	return LAYOUT_TO_PROFILE[self:GetLayout()] or format("SixUI %s", self:GetLayout())
end

-- Check whether a layout is active.
function M:IsLayoutActive(layout)
	return self:GetLayout() == layout
end

-- Check whether a layout is known.
function M:IsLayoutKnown(layout)
	return self:GetOptions()[layout] ~= nil
end

-- Check whether the SetLayout method is locked.
function M:IsSetLayoutLocked()
	return not not self.mutex
end

-- Lock the SetLayout method.
function M:LockSetLayout()
	self.mutex = true
end

-- Set the active layout.
function M:SetLayout(layout, force)
	if not self:IsSetLayoutLocked() then
		if not self:IsLayoutKnown(layout) then
			A.PrintF("The layout '%s' wasn't recognised.", A.Highlight(layout))
		elseif not force and self:IsLayoutActive(layout) then
			A.PrintF("The layout '%s' is already active.", A.Highlight(layout))
		else
			D.db.profile.layout.activeLayout = layout
			A:GetModule("Core Setup"):SetProfile()
			self:SendMessage("SIXUI_LAYOUT_CHANGED", layout)
		end
	end
end

-- Unlock the SetLayout method.
function M:UnlockSetLayout()
	self.mutex = false
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:PLAYER_SPECIALIZATION_CHANGED(event)
	local layoutInfo = EditModeManagerFrame:GetActiveLayoutInfo()
	local layout = PROFILE_TO_LAYOUT[layoutInfo.layoutName]
	
	if layoutInfo.layoutType == Enum.EditModeLayoutType.Preset then
		A.SetEditModeLayout(self:GetProfile())
	elseif layout then
		self:SetLayout(layout, true)
	end
end

function M:SIXUI_PROFILE_UPDATE(event)
	self:SetLayout(self:GetLayout(), true)
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

function M:C_EditMode_SetActiveLayout()
	local layoutInfo = EditModeManagerFrame:GetActiveLayoutInfo()
	local layout = PROFILE_TO_LAYOUT[layoutInfo.layoutName]
	
	if layout then
		self:SetLayout(layout, true)
	end
end
